@@ -64,7 +64,7 @@
 #'   The confidence bounds reflect uncertainty about the initial state (but
 #'   currently not about the parameter estimates) if initial values are not
 #'   specified. If initial values are specified, there won't currently be
-#'   confidence intervals. See the cited paper by Virolainen (2020) for details
+#'   confidence intervals. See the cited paper by Virolainen (2021) for details
 #'   about the algorithm.
 #'
 #'   Note that if the argument \code{scale} is used, the scaled responses of
@@ -76,6 +76,10 @@
 #'   element) and confidence intervals in \code{$conf_ints} (the second
 #'   element). The first row is for the GIRF at impact \eqn{(n=0)}, the second
 #'   for \eqn{n=1}, the third for \eqn{n=2}, and so on.
+#'
+#'   The element \code{$all_girfs} is a list containing results from all the individual GIRFs
+#'   obtained from the MC repetitions. Each element is for one shock and results are in
+#'   array of the form \code{[horizon, variables, MC-repetitions]}.
 #' @seealso \code{\link{GFEVD}}, \code{\link{fitGSMVAR}}, \code{\link{GSMVAR}},
 #'   \code{\link{gsmvar_to_sgsmvar}}, \code{\link{reorder_W_columns}},
 #'   \code{\link{swap_W_signs}}, \code{\link{simulate.gsmvar}},
@@ -205,6 +209,8 @@ GIRF <- function(gsmvar, which_shocks, shock_size=1, N=30, R1=250, R2=250, init_
   parallel::stopCluster(cl=cl)
 
   GIRF_results <- vector("list", length=length(which_shocks))
+  all_GIRFS <- vector("list", length=length(which_shocks))
+  names(all_GIRFS) <- paste0("shock", which_shocks)
   names(GIRF_results) <- paste0("shock", which_shocks)
 
   for(i1 in 1:length(which_shocks)) { # Go through shocks
@@ -236,6 +242,7 @@ GIRF <- function(gsmvar, which_shocks, shock_size=1, N=30, R1=250, R2=250, init_
         res_in_array[, , i2] <- one_scale*res_in_array[, , i2]
       }
     }
+    all_GIRFS[[i1]] <- res_in_array
 
     # Point estimates, confidence intervals
     colnames(res_in_array) <- colnames(GIRF_shocks[[1]][[1]])
@@ -253,6 +260,7 @@ GIRF <- function(gsmvar, which_shocks, shock_size=1, N=30, R1=250, R2=250, init_
 
   cat("Finished!\n")
   ret <- structure(list(girf_res=GIRF_results,
+                        all_girfs=all_GIRFS,
                         shocks=which_shocks,
                         shock_size=shock_size,
                         N=N,
@@ -307,7 +315,7 @@ GIRF <- function(gsmvar, which_shocks, shock_size=1, N=30, R1=250, R2=250, init_
 #'
 #'   The GFEVD is a forecast error variance decomposition calculated with the generalized impulse response function (GIRF).
 #'   Lanne and Nyberg (2016) for details. Note, however, that the related GIRFs are calculated using the algorithm given in
-#'   Virolainen (2020).
+#'   Virolainen (2021).
 #' @return Returns and object of class 'gfevd' containing the GFEVD for all the variables and if
 #'   \code{include_mixweights=TRUE} also to the mixing weights. Note that the decomposition does not
 #'   exist at horizon zero for mixing weights because the related GIRFs are always zero at impact.
@@ -320,7 +328,7 @@ GIRF <- function(gsmvar, which_shocks, shock_size=1, N=30, R1=250, R2=250, init_
 #'      and Nonlineae Multivariate Models. \emph{Oxford Bulletin of Economics and Statistics}, \strong{78}, 4, 595-603.
 #'    \item Kalliovirta L., Meitz M. and Saikkonen P. 2016. Gaussian mixture vector autoregression.
 #'          \emph{Journal of Econometrics}, \strong{192}, 485-498.
-#'    \item Virolainen S. 2020. Structural Gaussian mixture vector autoregressive model. Unpublished working
+#'    \item Virolainen S. 2021. Structural Gaussian mixture vector autoregressive model. Unpublished working
 #'      paper, available as arXiv:2007.04713.
 #'    \item Virolainen S. 2021. Gaussian and Student's t mixture vector autoregressive model. Unpublished working
 #'      paper, available as arXiv:2109.13648.
